using module VMware.PowerCLI.VCenter.Types.CertificateManagement
using namespace System.Collections.Generic
using namespace VMware.VimAutomation.ViCore.Types.V1
using namespace VMware.VimAutomation.Sdk.Util10Ps.BaseCmdlet

. (Join-Path $PSScriptRoot "../utils/Connection.ps1")
. (Join-Path $PSScriptRoot "../utils/ConvertTo-PemCertificate.ps1")
. (Join-Path $PSScriptRoot "../utils/ConvertTo-X509Certificate.ps1")

<#
.SYNOPSIS
This cmdlet removes one or more certificates or certificate chains from the vCenter Server or ESXi trusted stores.

.DESCRIPTION
This cmdlet removes one or more certificates or certificate chains from the vCenter Server or ESXi trusted stores.

.PARAMETER VITrustedCertificate
Specifies one or more certificate/entity object(s) of the certificate(s) you want to remove.

Note: You must use the Get-VITrustedCertificate cmdlet to obtain one or more certificate/entity object(s). The object returned by Get-VITrustedCertificate is a pair of the certificate and the vCenter Server or ESXi entity that trusts the certificate.

.PARAMETER Server
Specifies the vCenter Server systems on which you want to run the cmdlet.
If no value is provided or $null value is passed to this parameter, the command runs on the default server.
For more information about default servers, see the description of the Connect-VIServer cmdlet.

.EXAMPLE
PS C:\> Get-VITrustedCertificate -VMHost "MyESXi" | `
    Where-Object { $_.Certificate.Thumbprint -eq "6B953A0738FD...4BD263BEB0" } | `
    Remove-VITrustedCertificate

Removes a certificate with the thumbprint "6B953A0738FD...4BD263BEB0" from the ESXi host called "MyESXi".

.INPUTS
VMware.PowerCLI.VCenter.Types.CertificateManagement.TrustedCertificateInfo

.OUTPUTS
None

.LINK

https://developer.vmware.com/docs/powercli/latest/vmware.powercli.vcenter/commands/remove-vitrustedcertificate


#>
function Remove-VITrustedCertificate {
    [CmdletBinding(
       ConfirmImpact = "High",
       SupportsShouldProcess = $true)]
    [OutputType([void])]
    Param (
        [Parameter(
            Mandatory = $true,
            ParameterSetName = "ByObject",
            Position = 0,
            ValueFromPipeline = $true)]
        [ValidateNotNull()]
        [TrustedCertificateInfo[]] $VITrustedCertificate
    )

    Process {
        $serverGroups = Group-ByServer $VITrustedCertificate
        foreach($connectionId in $serverGroups.Keys) {
            [VIServer] $activeServer = Get-ServerByUid $connectionId

            $apiServer = GetApiServer($activeServer)

            $serverObjects = $serverGroups[$connectionId]
            $certsToRemoveFromVCenter = $serverObjects | Where-Object { $_.EntityType -eq [CertificateEntityType]::VCenter }
            $certsToRemoveFromEsx = $serverObjects | Where-Object { $_.EntityType -eq [CertificateEntityType]::EsxHost }
            $unsupportedCertificates = $serverObjects | Where-Object { ($_.EntityType -ne [CertificateEntityType]::VCenter) -and ($_.EntityType -ne [CertificateEntityType]::EsxHost) }

            if($certsToRemoveFromVCenter) {
                ValidateApiVersionSupported -server $activeServer -major 7 -minor 0
                foreach($certToRemove in $certsToRemoveFromVCenter) {
                    $trustChainId = $UidUtil.GetValue($certToRemove.Uid, "ViTrustedCertificate")

                    $description = "the certificate with subject '$($certToRemove.Subject)' and thumbprint '$($certToRemove.Certificate.Thumbprint)' from the trust store of vCenter '$($certToRemove.TrustedByEntity.Name)'"
                    $shouldProcessDescription = "Removing $description"
                    $shouldProcessWarning = "Are you sure you want to remove $description"

                    if($PSCmdlet.ShouldProcess(
                        $shouldProcessDescription,
                        $shouldProcessWarning,
                        "Remove certificate from trusted store")) {

                        try {
                            Invoke-DeleteChainCertificateManagementTrustedRootChains `
                                -Chain $trustChainId `
                                -Server $apiServer `
                                -Confirm:$false `
                                -ErrorAction:Stop
                        } catch {
                            Write-PowerCLIError `
                                -ErrorObject $_ `
                                -ErrorId "PowerCLI_VITrustedCertificate_DeleteChainCertificateManagementTrustedRootChains"
                        }
                    }
                }
            }

            if($certsToRemoveFromEsx) {
                $mapEsxToCertsToRemove = @{}

                # Grouping by ESX
                foreach($certToRemove in $certsToRemoveFromEsx) {
                    [List[object]] $group = $null
                    if($mapEsxToCertsToRemove.Contains($certToRemove.TrustedByEntity.Id)) {
                        $group = $mapEsxToCertsToRemove[$certToRemove.TrustedByEntity.Id]
                    } else {
                        $group = [List[object]]::new()
                        $mapEsxToCertsToRemove[$certToRemove.TrustedByEntity.Id] = $group
                    }

                    $group.Add($certToRemove)
                }

                foreach($vmHostId in $mapEsxToCertsToRemove.Keys) {
                    $certificatesToRemove = $mapEsxToCertsToRemove[$vmHostId]

                    try {
                        $currentVMHost = Get-VMHost -Id $vmHostId -Server $activeServer -ErrorAction:Stop
                        $certificateManager = Get-View $currentVMHost.ExtensionData.ConfigManager.CertificateManager `
                            -Server $activeServer -ErrorAction:Stop

                        foreach($userObject in $certificatesToRemove) {
                            [string[]] $currentCertificatePems = $certificateManager.ListCACertificates()
                            if($null -eq $currentCertificatePems) {
                                $currentCertificatePems = [string[]]::new(0)
                            }
                            $certThumbprint = $userObject.Certificate.Thumbprint

                            $found = $null
                            for($i = 0; $i -lt $currentCertificatePems.Length; $i++) {
                                $certObject = $currentCertificatePems[$i] | ConvertTo-X509Certificate
                                if($certObject.Thumbprint -eq $certThumbprint) {
                                    $found = $currentCertificatePems[$i]
                                    break
                                }
                            }

                            if(-not $found) {
                                Write-PowerCLIError `
                                    -ErrorId "PowerCLI_VITrustedCertificate_NotFoundInEsxTrustStore" `
                                    "Certificate with thumbprint '$certThumbprint' in the trusted store of ESX host '$($currentVMHost.Name)' not found."
                                continue
                            }

                            $description = "the certificate with subject '$($userObject.Subject)' and thumbprint '$($userObject.Certificate.Thumbprint)' from the trust store of ESX '$($userObject.TrustedByEntity.Name)'"
                            $shouldProcessDescription = "Removing $description"
                            $shouldProcessWarning = "Are you sure you want to remove $description"
                            if($PSCmdlet.ShouldProcess(
                                $shouldProcessDescription,
                                $shouldProcessWarning,
                                "Remove certificate from trusted store")) {

                                try {
                                    $updatedCertificatePems = [List[string]]::new($currentCertificatePems)
                                    $updatedCertificatePems.Remove($found) | Out-Null
                                    $certificateManager.ReplaceCACertificatesAndCRLs([string[]] $updatedCertificatePems, $null) | Out-Null
                                } catch {
                                    Write-PowerCLIError `
                                        -ErrorObject $_ `
                                        -ErrorId "PowerCLI_VITrustedCertificate_ReplaceCACertificatesAndCRLs"
                                }
                            }
                        }
                    } catch {
                        Write-PowerCLIError `
                            -ErrorObject $_ `
                            -ErrorId "PowerCLI_VITrustedCertificate_Remove"
                    }
                }
            }

            if($unsupportedCertificates) {
                $unsupportedCertificates |
                    Write-PowerCLIError -ErrorId "PowerCLI_VITrustedCertificate_RemoveUnsupported" `
                        "Enity type '$($_.EntityType)' for certificate '$($_.Subject)' trusted by '$($_.TrustedByEntity)' is not supported by this command."
            }
        }
    }
}

# SIG # Begin signature block
# MIIexgYJKoZIhvcNAQcCoIIetzCCHrMCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAvYfLZKkIh5mWH
# YYyqhsACxLl0yscR8wID+b2M1wJ4DKCCDdowggawMIIEmKADAgECAhAIrUCyYNKc
# TJ9ezam9k67ZMA0GCSqGSIb3DQEBDAUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNV
# BAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBHNDAeFw0yMTA0MjkwMDAwMDBaFw0z
# NjA0MjgyMzU5NTlaMGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwg
# SW5jLjFBMD8GA1UEAxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcg
# UlNBNDA5NiBTSEEzODQgMjAyMSBDQTEwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAw
# ggIKAoICAQDVtC9C0CiteLdd1TlZG7GIQvUzjOs9gZdwxbvEhSYwn6SOaNhc9es0
# JAfhS0/TeEP0F9ce2vnS1WcaUk8OoVf8iJnBkcyBAz5NcCRks43iCH00fUyAVxJr
# Q5qZ8sU7H/Lvy0daE6ZMswEgJfMQ04uy+wjwiuCdCcBlp/qYgEk1hz1RGeiQIXhF
# LqGfLOEYwhrMxe6TSXBCMo/7xuoc82VokaJNTIIRSFJo3hC9FFdd6BgTZcV/sk+F
# LEikVoQ11vkunKoAFdE3/hoGlMJ8yOobMubKwvSnowMOdKWvObarYBLj6Na59zHh
# 3K3kGKDYwSNHR7OhD26jq22YBoMbt2pnLdK9RBqSEIGPsDsJ18ebMlrC/2pgVItJ
# wZPt4bRc4G/rJvmM1bL5OBDm6s6R9b7T+2+TYTRcvJNFKIM2KmYoX7BzzosmJQay
# g9Rc9hUZTO1i4F4z8ujo7AqnsAMrkbI2eb73rQgedaZlzLvjSFDzd5Ea/ttQokbI
# YViY9XwCFjyDKK05huzUtw1T0PhH5nUwjewwk3YUpltLXXRhTT8SkXbev1jLchAp
# QfDVxW0mdmgRQRNYmtwmKwH0iU1Z23jPgUo+QEdfyYFQc4UQIyFZYIpkVMHMIRro
# OBl8ZhzNeDhFMJlP/2NPTLuqDQhTQXxYPUez+rbsjDIJAsxsPAxWEQIDAQABo4IB
# WTCCAVUwEgYDVR0TAQH/BAgwBgEB/wIBADAdBgNVHQ4EFgQUaDfg67Y7+F8Rhvv+
# YXsIiGX0TkIwHwYDVR0jBBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0P
# AQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHcGCCsGAQUFBwEBBGswaTAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAC
# hjVodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9v
# dEc0LmNydDBDBgNVHR8EPDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5j
# b20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNybDAcBgNVHSAEFTATMAcGBWeBDAED
# MAgGBmeBDAEEATANBgkqhkiG9w0BAQwFAAOCAgEAOiNEPY0Idu6PvDqZ01bgAhql
# +Eg08yy25nRm95RysQDKr2wwJxMSnpBEn0v9nqN8JtU3vDpdSG2V1T9J9Ce7FoFF
# UP2cvbaF4HZ+N3HLIvdaqpDP9ZNq4+sg0dVQeYiaiorBtr2hSBh+3NiAGhEZGM1h
# mYFW9snjdufE5BtfQ/g+lP92OT2e1JnPSt0o618moZVYSNUa/tcnP/2Q0XaG3Ryw
# YFzzDaju4ImhvTnhOE7abrs2nfvlIVNaw8rpavGiPttDuDPITzgUkpn13c5Ubdld
# AhQfQDN8A+KVssIhdXNSy0bYxDQcoqVLjc1vdjcshT8azibpGL6QB7BDf5WIIIJw
# 8MzK7/0pNVwfiThV9zeKiwmhywvpMRr/LhlcOXHhvpynCgbWJme3kuZOX956rEnP
# LqR0kq3bPKSchh/jwVYbKyP/j7XqiHtwa+aguv06P0WmxOgWkVKLQcBIhEuWTatE
# QOON8BUozu3xGFYHKi8QxAwIZDwzj64ojDzLj4gLDb879M4ee47vtevLt/B3E+bn
# KD+sEq6lLyJsQfmCXBVmzGwOysWGw/YmMwwHS6DTBwJqakAwSEs0qFEgu60bhQji
# WQ1tygVQK+pKHJ6l/aCnHwZ05/LWUpD9r4VIIflXO7ScA+2GRfS0YW6/aOImYIbq
# yK+p/pQd52MbOoZWeE4wggciMIIFCqADAgECAhAOxvKydqFGoH0ObZNXteEIMA0G
# CSqGSIb3DQEBCwUAMGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwg
# SW5jLjFBMD8GA1UEAxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcg
# UlNBNDA5NiBTSEEzODQgMjAyMSBDQTEwHhcNMjEwODEwMDAwMDAwWhcNMjMwODEw
# MjM1OTU5WjCBhzELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEjAQ
# BgNVBAcTCVBhbG8gQWx0bzEVMBMGA1UEChMMVk13YXJlLCBJbmMuMRUwEwYDVQQD
# EwxWTXdhcmUsIEluYy4xITAfBgkqhkiG9w0BCQEWEm5vcmVwbHlAdm13YXJlLmNv
# bTCCAaIwDQYJKoZIhvcNAQEBBQADggGPADCCAYoCggGBAMD6lJG8OWkM12huIQpO
# /q9JnhhhW5UyW9if3/UnoFY3oqmp0JYX/ZrXogUHYXmbt2gk01zz2P5Z89mM4gqR
# bGYC2tx+Lez4GxVkyslVPI3PXYcYSaRp39JsF3yYifnp9R+ON8O3Gf5/4EaFmbeT
# ElDCFBfExPMqtSvPZDqekodzX+4SK1PIZxCyR3gml8R3/wzhb6Li0mG7l0evQUD0
# FQAbKJMlBk863apeX4ALFZtrnCpnMlOjRb85LsjV5Ku4OhxQi1jlf8wR+za9C3DU
# ki60/yiWPu+XXwEUqGInIihECBbp7hfFWrnCCaOgahsVpgz8kKg/XN4OFq7rbh4q
# 5IkTauqFhHaE7HKM5bbIBkZ+YJs2SYvu7aHjw4Z8aRjaIbXhI1G+NtaNY7kSRrE4
# fAyC2X2zV5i4a0AuAMM40C1Wm3gTaNtRTHnka/pbynUlFjP+KqAZhOniJg4AUfjX
# sG+PG1LH2+w/sfDl1A8liXSZU1qJtUs3wBQFoSGEaGBeDQIDAQABo4ICJTCCAiEw
# HwYDVR0jBBgwFoAUaDfg67Y7+F8Rhvv+YXsIiGX0TkIwHQYDVR0OBBYEFIhC+HL9
# QlvsWsztP/I5wYwdfCFNMB0GA1UdEQQWMBSBEm5vcmVwbHlAdm13YXJlLmNvbTAO
# BgNVHQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUHAwMwgbUGA1UdHwSBrTCB
# qjBToFGgT4ZNaHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3Rl
# ZEc0Q29kZVNpZ25pbmdSU0E0MDk2U0hBMzg0MjAyMUNBMS5jcmwwU6BRoE+GTWh0
# dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNENvZGVTaWdu
# aW5nUlNBNDA5NlNIQTM4NDIwMjFDQTEuY3JsMD4GA1UdIAQ3MDUwMwYGZ4EMAQQB
# MCkwJwYIKwYBBQUHAgEWG2h0dHA6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzCBlAYI
# KwYBBQUHAQEEgYcwgYQwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0
# LmNvbTBcBggrBgEFBQcwAoZQaHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0Rp
# Z2lDZXJ0VHJ1c3RlZEc0Q29kZVNpZ25pbmdSU0E0MDk2U0hBMzg0MjAyMUNBMS5j
# cnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAgEACQAYaQI6Nt2KgxdN
# 6qqfcHB33EZRSXkvs8O9iPZkdDjEx+2fgbBPLUvk9A7T8mRw7brbcJv4PLTYJDFo
# c5mlcmG7/5zwTOuIs2nBGXc/uxCnyW8p7kD4Y0JxPKEVQoIQ8lJS9Uy/hBjyakeV
# ef982JyzvDbOlLBy6AS3ZpXVkRY5y3Va+3v0R/0xJ+JRxUicQhiZRidq2TCiWEas
# d+tLL6jrKaBO+rmP52IM4eS9d4Yids7ogKEBAlJi0NbvuKO0CkgOlFjp1tOvD4sQ
# taHIMmqi40p4Tjyf/sY6yGjROXbMeeF1vlwbBAASPWpQuEIxrNHoVN30YfJyuOWj
# zdiJUTpeLn9XdjM3UlhfaHP+oIAKcmkd33c40SFRlQG9+P9Wlm7TcPxGU4wzXI8n
# Cw/h235jFlAAiWq9L2r7Un7YduqsheJVpGoXmRXJH0T2G2eNFS5/+2sLn98kN2Cn
# J7j6C242onjkZuGL2/+gqx8m5Jbpu9P4IAeTC1He/mX9j6XpIu+7uBoRVwuWD1i0
# N5SiUz7Lfnbr6Q1tHMXKDLFdwVKZos2AKEZhv4SU0WvenMJKDgkkhVeHPHbTahQf
# P1MetR8tdRs7uyTWAjPK5xf5DLEkXbMrUkpJ089fPvAGVHBcHRMqFA5egexOb6sj
# tKncUjJ1xAAtAExGdCh6VD2U5iYxghBCMIIQPgIBATB9MGkxCzAJBgNVBAYTAlVT
# MRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UEAxM4RGlnaUNlcnQgVHJ1
# c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEzODQgMjAyMSBDQTECEA7G
# 8rJ2oUagfQ5tk1e14QgwDQYJYIZIAWUDBAIBBQCggZYwGQYJKoZIhvcNAQkDMQwG
# CisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwKgYKKwYB
# BAGCNwIBDDEcMBqhGIAWaHR0cDovL3d3dy52bXdhcmUuY29tLzAvBgkqhkiG9w0B
# CQQxIgQglKp9K+cGeDpuvOB67F+iqetraGRbFTlo5kP0nGtQkdQwDQYJKoZIhvcN
# AQEBBQAEggGATtt6Y7T2sNSH1hr43Y6OpmodL3J+aleY/nRlu2VgX3mebB2PkN3K
# LD9KbAYCkvxwQY4ABdoz5DnisZ997QBNNqF2DnyUPRk9oJAyiE8uSMSJm2oqsXT8
# dqFGZjwONPMfBvFxOs9bnqMAuVCSyEpcWbBtJaioXjoCgoMPS82tSXCkeaYnpkkg
# jHug2T4OPa9y+o67JtXPMxFiHz5ciAD2XDE87teUcYIoWNHQf0YhImkY2GOmTnaP
# 8syefvkDNDRgc5LHg7tClAV7z5GbmYQ+EABpjX1zmEjfZCH7rip0Df+Ys1nHGT+I
# 5SyW1FE1w1ZSh5b6HiEotdQ21xRuAqeN6YhxpOIy/9lmKpBxx4lclCK0BZgFl5eE
# 6OJkCWo/sRwNDBTHe+XXZc95mRX5mwZ04yD6t6WhOBGphMkBrQ5IfIojahSWZmwU
# TUptOqFmZJHOAnWGQqPx5vjbQ/lgEIbY0OQ/3vlnc9Y8PAaHNvDa/XhZb0rZOiig
# BgDyRLIiP7RsoYINfTCCDXkGCisGAQQBgjcDAwExgg1pMIINZQYJKoZIhvcNAQcC
# oIINVjCCDVICAQMxDzANBglghkgBZQMEAgEFADB3BgsqhkiG9w0BCRABBKBoBGYw
# ZAIBAQYJYIZIAYb9bAcBMDEwDQYJYIZIAWUDBAIBBQAEIKuzK/GfsjDfwG4R4FsV
# y0NfDLMSuLkcrllSJjhz0uI9AhA5H9e36LdYJ4hPI9+1FP18GA8yMDIxMDkxNDIz
# MDQ1N1qgggo3MIIE/jCCA+agAwIBAgIQDUJK4L46iP9gQCHOFADw3TANBgkqhkiG
# 9w0BAQsFADByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkw
# FwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEy
# IEFzc3VyZWQgSUQgVGltZXN0YW1waW5nIENBMB4XDTIxMDEwMTAwMDAwMFoXDTMx
# MDEwNjAwMDAwMFowSDELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJ
# bmMuMSAwHgYDVQQDExdEaWdpQ2VydCBUaW1lc3RhbXAgMjAyMTCCASIwDQYJKoZI
# hvcNAQEBBQADggEPADCCAQoCggEBAMLmYYRnxYr1DQikRcpja1HXOhFCvQp1dU2U
# tAxQtSYQ/h3Ib5FrDJbnGlxI70Tlv5thzRWRYlq4/2cLnGP9NmqB+in43Stwhd4C
# GPN4bbx9+cdtCT2+anaH6Yq9+IRdHnbJ5MZ2djpT0dHTWjaPxqPhLxs6t2HWc+xO
# bTOKfF1FLUuxUOZBOjdWhtyTI433UCXoZObd048vV7WHIOsOjizVI9r0TXhG4wOD
# MSlKXAwxikqMiMX3MFr5FK8VX2xDSQn9JiNT9o1j6BqrW7EdMMKbaYK02/xWVLwf
# oYervnpbCiAvSwnJlaeNsvrWY4tOpXIc7p96AXP4Gdb+DUmEvQECAwEAAaOCAbgw
# ggG0MA4GA1UdDwEB/wQEAwIHgDAMBgNVHRMBAf8EAjAAMBYGA1UdJQEB/wQMMAoG
# CCsGAQUFBwMIMEEGA1UdIAQ6MDgwNgYJYIZIAYb9bAcBMCkwJwYIKwYBBQUHAgEW
# G2h0dHA6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAfBgNVHSMEGDAWgBT0tuEgHf4p
# rtLkYaWyoiWyyBc1bjAdBgNVHQ4EFgQUNkSGjqS6sGa+vCgtHUQ23eNqerwwcQYD
# VR0fBGowaDAyoDCgLoYsaHR0cDovL2NybDMuZGlnaWNlcnQuY29tL3NoYTItYXNz
# dXJlZC10cy5jcmwwMqAwoC6GLGh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9zaGEy
# LWFzc3VyZWQtdHMuY3JsMIGFBggrBgEFBQcBAQR5MHcwJAYIKwYBBQUHMAGGGGh0
# dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBPBggrBgEFBQcwAoZDaHR0cDovL2NhY2Vy
# dHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0U0hBMkFzc3VyZWRJRFRpbWVzdGFtcGlu
# Z0NBLmNydDANBgkqhkiG9w0BAQsFAAOCAQEASBzctemaI7znGucgDo5nRv1CclF0
# CiNHo6uS0iXEcFm+FKDlJ4GlTRQVGQd58NEEw4bZO73+RAJmTe1ppA/2uHDPYuj1
# UUp4eTZ6J7fz51Kfk6ftQ55757TdQSKJ+4eiRgNO/PT+t2R3Y18jUmmDgvoaU+2Q
# zI2hF3MN9PNlOXBL85zWenvaDLw9MtAby/Vh/HUIAHa8gQ74wOFcz8QRcucbZEnY
# Ipp1FUL1LTI4gdr0YKK6tFL7XOBhJCVPst/JKahzQ1HavWPWH1ub9y4bTxMd90oN
# cX6Xt/Q/hOvB46NJofrOp79Wz7pZdmGJX36ntI5nePk2mOHLKNpbh6aKLzCCBTEw
# ggQZoAMCAQICEAqhJdbWMht+QeQF2jaXwhUwDQYJKoZIhvcNAQELBQAwZTELMAkG
# A1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRp
# Z2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENB
# MB4XDTE2MDEwNzEyMDAwMFoXDTMxMDEwNzEyMDAwMFowcjELMAkGA1UEBhMCVVMx
# FTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNv
# bTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIFRpbWVzdGFtcGlu
# ZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAL3QMu5LzY9/3am6
# gpnFOVQoV7YjSsQOB0UzURB90Pl9TWh+57ag9I2ziOSXv2MhkJi/E7xX08PhfgjW
# ahQAOPcuHjvuzKb2Mln+X2U/4Jvr40ZHBhpVfgsnfsCi9aDg3iI/Dv9+lfvzo7oi
# PhisEeTwmQNtO4V8CdPuXciaC1TjqAlxa+DPIhAPdc9xck4Krd9AOly3UeGheRTG
# TSQjMF287DxgaqwvB8z98OpH2YhQXv1mblZhJymJhFHmgudGUP2UKiyn5HU+upgP
# hH+fMRTWrdXyZMt7HgXQhBlyF/EXBu89zdZN7wZC/aJTKk+FHcQdPK/P2qwQ9d2s
# rOlW/5MCAwEAAaOCAc4wggHKMB0GA1UdDgQWBBT0tuEgHf4prtLkYaWyoiWyyBc1
# bjAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzASBgNVHRMBAf8ECDAG
# AQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDCDB5Bggr
# BgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNv
# bTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lD
# ZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHoweDA6oDigNoY0aHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDA6
# oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElE
# Um9vdENBLmNybDBQBgNVHSAESTBHMDgGCmCGSAGG/WwAAgQwKjAoBggrBgEFBQcC
# ARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzALBglghkgBhv1sBwEwDQYJ
# KoZIhvcNAQELBQADggEBAHGVEulRh1Zpze/d2nyqY3qzeM8GN0CE70uEv8rPAwL9
# xafDDiBCLK938ysfDCFaKrcFNB1qrpn4J6JmvwmqYN92pDqTD/iy0dh8GWLoXoIl
# HsS6HHssIeLWWywUNUMEaLLbdQLgcseY1jxk5R9IEBhfiThhTWJGJIdjjJFSLK8p
# ieV4H9YLFKWA1xJHcLN11ZOFk362kmf7U2GJqPVrlsD0WGkNfMgBsbkodbeZY4Ui
# jGHKeZR+WfyMD+NvtQEmtmyl7odRIeRYYJu6DC0rbaLEfrvEJStHAgh8Sa4TtuF8
# QkIoxhhWz0E0tmZdtnR79VYzIi8iNrJLokqV2PWmjlIxggKGMIICggIBATCBhjBy
# MQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3
# d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQg
# SUQgVGltZXN0YW1waW5nIENBAhANQkrgvjqI/2BAIc4UAPDdMA0GCWCGSAFlAwQC
# AQUAoIHRMBoGCSqGSIb3DQEJAzENBgsqhkiG9w0BCRABBDAcBgkqhkiG9w0BCQUx
# DxcNMjEwOTE0MjMwNDU3WjArBgsqhkiG9w0BCRACDDEcMBowGDAWBBTh14Ko4ZG+
# 72vKFpG1qrSUpiSb8zAvBgkqhkiG9w0BCQQxIgQg6x1zz41yh2VqJo88gQsBdMZP
# t5iP6jZ8N8zRpgGkITMwNwYLKoZIhvcNAQkQAi8xKDAmMCQwIgQgsxCQBrwK2YMH
# kVcp4EQDQVyD4ykrYU8mlkyNNXHs9akwDQYJKoZIhvcNAQEBBQAEggEAQQqOQUP1
# BTwsbA48xhT/eE37GtVK50a1ZuZcbTrq7hHtzaI260ArWfL83WL3f6TCCnIC0Lc1
# WXslG3cGGZDz0kzcZr+20TlibV1e7lvrx4TvvdDBYRrn0ZQHHnNzzyTNjLQZRwXx
# HsTByFB+fH0yKDhjndkMRqBlrH6U/jl5UKfYklXzmC9gt7jP5ibr5NHjrIecrfzi
# 1oVn58jFYY+Ji6tyyEIDyp0SFi8KauLea3AduhIS1gP4L+LgH9IlSR7UBCskyGXp
# 76PUbKMydT0eDaZtfR7BPDAKid3qQPeoQ7etAFhRRAXq6aq14qfvGKZsK+R36wi3
# mGorCllP/teo6A==
# SIG # End signature block
